@@ -15,13 +15,12 @@ import serial_asyncio
 from . import requests, responses
 from .constants import DATA_MIXERS, DEFAULT_IP, DEFAULT_NETMASK, WLAN_ENCRYPTION
 from .devices import ECOMAX_ADDRESS, DevicesCollection
-from .exceptions import FrameError, FrameTypeError
+from .exceptions import ConnectionFailedError, FrameError, FrameTypeError
 from .frame import Frame
 from .stream import FrameReader, FrameWriter
 
 _LOGGER = logging.getLogger(__name__)
 
-READER_TIMEOUT: Final = 5
 CONNECT_TIMEOUT: Final = 5
 RECONNECT_TIMEOUT: Final = 20
 
@@ -36,6 +35,7 @@ class Connection(ABC):
         _net -- network information for device available message
         _devices -- collection of all available devices
         _callback_task -- callback task reference
+        _callback_closed -- callback to call when connection is closed
     """
 
     def __init__(self, **kwargs):
@@ -50,6 +50,7 @@ class Connection(ABC):
         self._net = {}
         self._devices = DevicesCollection()
         self._callback_task = None
+        self._callback_closed = None
 
     def __enter__(self):
         """Provides entry point for context manager."""
@@ -71,8 +72,10 @@ class Connection(ABC):
             interval -- update interval in seconds
         """
         while True:
-            if not self.closed:
+            try:
                 await callback(self._devices, self)
+            except Exception:  # pylint: disable=broad-except
+                pass
 
             await asyncio.sleep(interval)
 
@@ -94,7 +97,6 @@ class Connection(ABC):
         elif isinstance(frame, responses.UID):
             device.uid = frame.data["UID"]
             device.product = frame.data["reg_name"]
-            writer.queue(requests.Password(recipient=frame.sender))
 
         elif isinstance(frame, responses.Password):
             device.password = frame.data
@@ -128,14 +130,13 @@ class Connection(ABC):
         while True:
             if self.closing:
                 await self.writer.close()
-                self.writer = None
-                self.closing = False
+                await self._close()
                 return False
 
             try:
-                frame = await asyncio.wait_for(reader.read(), timeout=READER_TIMEOUT)
+                frame = await reader.read()
             except FrameTypeError as e:
-                _LOGGER.info("Type error: %s", e)
+                _LOGGER.debug("Type error: %s", e)
             except FrameError as e:
                 _LOGGER.warning("Frame error: %s", e)
             else:
@@ -157,13 +158,13 @@ class Connection(ABC):
             interval -- user-defined update interval in seconds
             reconnect_on_failure -- should we try reconnecting on failure
         """
-        self._callback_task = asyncio.create_task(self._callback(callback, interval))
         while True:
             try:
-                reader, self.writer = await asyncio.wait_for(
-                    self.connect(), timeout=CONNECT_TIMEOUT
-                )
+                reader, self.writer = await self.connect()
                 await self.writer.write(requests.StartMaster(recipient=ECOMAX_ADDRESS))
+                self._callback_task = asyncio.create_task(
+                    self._callback(callback, interval)
+                )
                 if not await self._read(reader):
                     break
             except (
@@ -172,17 +173,25 @@ class Connection(ABC):
                 ConnectionResetError,
                 OSError,
                 SerialException,
-            ) as e:
+            ) as connection_failure:
                 if not reconnect_on_failure:
-                    raise e
+                    raise ConnectionFailedError from connection_failure
 
                 _LOGGER.error(
                     "Connection to device failed, retrying in %i seconds...",
                     RECONNECT_TIMEOUT,
                 )
-                self.writer = None
+                await self._close()
                 await asyncio.sleep(RECONNECT_TIMEOUT)
 
+    async def _close(self) -> None:
+        """Declares connection as closed."""
+        await self.async_close()
+        self.writer = None
+        self.closing = False
+        if self._callback_closed is not None:
+            await self._callback_closed(self)
+
     def run(
         self,
         callback: Callable[[DevicesCollection, Connection], Awaitable[Any]],
@@ -253,13 +262,28 @@ class Connection(ABC):
         wlan["status"] = True
         self._net["wlan"] = wlan
 
+    def on_closed(
+        self, callback: Optional[Callable[[Connection], Awaitable[Any]]] = None
+    ) -> None:
+        """Sets callback to be called when connection is closed.
+
+        Keyword arguments:
+            callback -- user-defined callback method
+        """
+        self._callback_closed = callback
+
     def close(self) -> None:
-        """Closes opened connection."""
+        """Closes connection."""
         if not self.closing:
             self.closing = True
             if self._callback_task is not None:
                 self._callback_task.cancel()
 
+    async def async_close(self) -> None:
+        """Closes connection asynchronously."""
+        loop = asyncio.get_running_loop()
+        await loop.run_in_executor(None, self.close)
+
     @property
     def closed(self) -> bool:
         """Returns connection state."""
@@ -295,8 +319,9 @@ class TcpConnection(Connection):
 
     async def connect(self) -> Tuple[FrameReader, FrameWriter]:
         """Initializes connection and returns frame reader and writer."""
-        reader, writer = await asyncio.open_connection(
-            host=self.host, port=self.port, **self.kwargs
+        reader, writer = await asyncio.wait_for(
+            asyncio.open_connection(host=self.host, port=self.port, **self.kwargs),
+            timeout=CONNECT_TIMEOUT,
         )
         return FrameReader(reader), FrameWriter(writer)
 
@@ -326,12 +351,15 @@ class SerialConnection(Connection):
 
     async def connect(self) -> Tuple[FrameReader, FrameWriter]:
         """Initializes connection and returns frame reader and writer."""
-        reader, writer = await serial_asyncio.open_serial_connection(
-            url=self.device,
-            baudrate=self.baudrate,
-            bytesize=serial_asyncio.serial.EIGHTBITS,
-            parity=serial_asyncio.serial.PARITY_NONE,
-            stopbits=serial_asyncio.serial.STOPBITS_ONE,
-            **self.kwargs,
+        reader, writer = await asyncio.wait_for(
+            serial_asyncio.open_serial_connection(
+                url=self.device,
+                baudrate=self.baudrate,
+                bytesize=serial_asyncio.serial.EIGHTBITS,
+                parity=serial_asyncio.serial.PARITY_NONE,
+                stopbits=serial_asyncio.serial.STOPBITS_ONE,
+                **self.kwargs,
+            ),
+            timeout=CONNECT_TIMEOUT,
         )
         return FrameReader(reader), FrameWriter(writer)
