@@ -2,31 +2,42 @@
 
 from __future__ import annotations
 
-from typing import Dict, List
+from typing import Dict, Final, List, Type
 
 from .exceptions import FrameTypeError
 from .factory import FrameFactory
-from .frame import Request
+from .frame import BROADCAST_ADDRESS, Request
+
+DEFAULT_VERSION: Final = 0
 
 
 class FrameBucket:
     """Keeps track of frame versions and stores versioning data.
 
     Attributes:
+        _address -- address of device that contains this frame bucket
         versions -- dictionary containing frame versions
     """
 
-    def __init__(self, versions: Dict[int, int] = None):
+    def __init__(
+        self,
+        address: int = BROADCAST_ADDRESS,
+        versions: Dict[int, int] = None,
+        required: List[Type[Request]] = None,
+    ):
         """Created FrameBucket instance.
 
         Keyword arguments:
             versions -- dictionary containing frame versions
         """
-        self.versions = {}
+        self.versions: Dict[int, int] = {}
+        self._address = address
         self._queue: List[Request] = []
-
         if versions is not None:
-            self.versions = versions
+            self.fill(versions)
+
+        if required is not None:
+            self.fill({frame.type_: DEFAULT_VERSION for frame in required})
 
     def __len__(self) -> int:
         """Gets number of stored frame versions."""
@@ -35,6 +46,7 @@ class FrameBucket:
     def __repr__(self) -> str:
         """Returns serializable string representation."""
         return f"""FrameBucket(
+    address: {self._address},
     versions: {self.versions}
 )
 """
@@ -58,7 +70,7 @@ class FrameBucket:
             version -- new frame version to update to
         """
         try:
-            frame = FrameFactory().get_frame(type_=type_)
+            frame = FrameFactory().get_frame(type_=type_, recipient=self._address)
         except FrameTypeError:
             return None
 
