@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import asyncio
 from asyncio import StreamReader, StreamWriter
 from typing import Final, List, Optional
 
@@ -11,6 +12,8 @@ from .factory import FrameFactory
 from .frame import BROADCAST_ADDRESS, ECONET_ADDRESS, HEADER_SIZE, Frame, Request
 
 READER_BUFFER_SIZE: Final = 1000
+READER_TIMEOUT: Final = 5
+WRITER_TIMEOUT: Final = 5
 
 
 class FrameWriter:
@@ -42,21 +45,32 @@ class FrameWriter:
             frame -- Frame instance to add
         """
         for frame in frames:
-            if isinstance(frame, Frame):
+            if isinstance(frame, Frame) and not self.has(frame):
                 self._queue.append(frame)
 
     def is_empty(self) -> bool:
         """Checks if write queue is empty."""
-        return len(self._queue) == 0
+        return bool(self._queue)
+
+    def has(self, request: Request) -> bool:
+        """Checks if write queue contains specific request.
+
+        Keyword arguments:
+            request - request to look for
+        """
+        for frame in self._queue:
+            if frame.type_ == request.type_:
+                return True
+
+        return False
 
     def collect(self, requests: List[Request]) -> None:
         """Collects changed parameters and adds them to write queue.
 
         Keyword arguments:
-            parameters -- list of device parameters
+            requests -- list of changed parameters
         """
-        for request in requests:
-            self.queue(request)
+        self.queue(*requests)
 
     async def process_queue(self) -> None:
         """Processes top-most write request from the stack."""
@@ -71,12 +85,12 @@ class FrameWriter:
             frame -- Frame instance to add
         """
         self.writer.write(frame.bytes)
-        await self.writer.drain()
+        await asyncio.wait_for(self.writer.drain(), timeout=WRITER_TIMEOUT)
 
     async def close(self) -> None:
         """Closes stream writer."""
         self.writer.close()
-        await self.writer.wait_closed()
+        await asyncio.wait_for(self.writer.wait_closed(), timeout=WRITER_TIMEOUT)
 
 
 class FrameReader:
@@ -99,7 +113,9 @@ class FrameReader:
         """Attempts to read READER_BUFFER_SIZE bytes, find
         valid frame in it and return corresponding Frame instance.
         """
-        buffer = await self.reader.read(READER_BUFFER_SIZE)
+        buffer = await asyncio.wait_for(
+            self.reader.read(READER_BUFFER_SIZE), timeout=READER_TIMEOUT
+        )
 
         if len(buffer) >= HEADER_SIZE:
             header = buffer[0:HEADER_SIZE]
