@@ -31,9 +31,9 @@ def from_bytes(
     for index in range(first_parameter, total_parameters + first_parameter):
         parameter = util.unpack_parameter(message, offset)
         if parameter is not None:
-            mixer[f"{MIXER_PARAMS[index%mixers_number]}"] = parameter
+            mixer[f"{MIXER_PARAMS[index%total_parameters]}"] = parameter
 
-        if index % parameters_number == 0:
+        if (index + 1) % total_parameters == 0:
             data[DATA_MIXERS].append(mixer)
             mixer = {}
 
