@@ -1,5 +1,7 @@
 """Contains output flags structure parser."""
 
+from typing import Any, Dict, Tuple
+
 from pyplumio import util
 from pyplumio.constants import (
     DATA_CIRCULATION_PUMP_FLAG,
@@ -9,12 +11,14 @@ from pyplumio.constants import (
 )
 
 
-def from_bytes(message: bytearray, offset: int = 0, data: dict = None) -> (dict, int):
+def from_bytes(
+    message: bytearray, offset: int = 0, data: Dict[str, Any] = None
+) -> Tuple[Dict[str, Any], int]:
     """Parses frame message into usable data.
 
     Keyword arguments:
-    message -- ecoNET message
-    offset -- current data offset
+        message -- message bytes
+        offset -- current data offset
     """
     if data is None:
         data = {}
